/*Copyright (C) 2010-2012 Duncan Deveaux

This file is part of Hikou no mizu.

Hikou no mizu is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Hikou no mizu is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Hikou no mizu.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef DEF_PHYSICS_WORLD
#define DEF_PHYSICS_WORLD

#include <vector>
#include "Structs/Box.hpp"
#include "Structs/Vector.hpp"

#include "Graphics/Drawable/Polygon.hpp" //Used for debugging

class PhysicsObject;

class PhysicsWorld
{
    public:
        PhysicsWorld();
        PhysicsWorld(const Box &worldBox, float gravity, float airFriction);
        
        void update(float frameTime);
        void debug();
        
        void addObject(PhysicsObject &object);
        void removeObject(PhysicsObject &object);
        
        void pulseObject(PhysicsObject &object, float xPulse, float yPulse);
        std::vector<PhysicsObject*> &getObjects();
        
        void setWorldBox(const Box &worldBox);
        const Box &getWorldBox() const;
        
        void setGravityForce(float gravity);
        float getGravityForce() const;
        
        void setAirFriction(float airFriction);
        float getAirFriction() const;
    
    private:
        std::vector<PhysicsObject*> m_objectsList;
        std::vector<Vector> m_objectsPulse;
        
        Box m_worldBox; //World box
        float m_gravity; //Gravity force in pixels / second
        float m_airFriction; //Air friction in pixels / second
};

#endif
