/*Copyright (C) 2010-2015 Duncan Deveaux

This file is part of Hikou no mizu.

Hikou no mizu is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Hikou no mizu is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Hikou no mizu.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "Button.hpp"

Button::Button() : Widget(),
m_soundEngine(NULL), m_focusSound(NO_SOUND), m_clickSound(NO_SOUND)
{
    
}

Button::Button(const Sprite &inactive, const Sprite &focused) :
Widget(), m_inactiveBack(inactive), m_focusedBack(focused),
m_soundEngine(NULL), m_focusSound(NO_SOUND), m_clickSound(NO_SOUND)
{
    
}

void Button::draw()
{
    //Update matrix
    Drawable::pushMatrix();
    Drawable::updateMatrix();
    
    if (m_focused) m_focusedBack.draw();
    else m_inactiveBack.draw();
    
    Drawable::popMatrix();
}

bool Button::mouseClick(const Vector &mouseCoords)
{
    if ( Widget::contains( Widget::localCoords(mouseCoords) ) )
    {
        //Launch click sound
        if (m_soundEngine != NULL && m_clickSound != NO_SOUND)
            m_soundEngine->playSound(m_clickSound);
        
        return true;
    }
    
    return false;
}

bool Button::mouseMove(const Vector &mouseCoords)
{
    if ( Widget::contains( Widget::localCoords(mouseCoords) ) )
    {
        if (!isFocused())
        {
            //Launch focus sound
            if (m_soundEngine != NULL && m_focusSound != NO_SOUND)
                m_soundEngine->playSound(m_focusSound);
            
            setFocus(true);
        }
        
        return true;
    }
    
    setFocus(false);
    return false;
}

void Button::setInactiveBack(const Sprite &inactive)
{
    m_inactiveBack = inactive;
}

const Sprite &Button::getInactiveBack() const
{
    return m_inactiveBack;
}

void Button::setFocusedBack(const Sprite &focused)
{
    m_focusedBack = focused;
}

const Sprite &Button::getFocusedBack() const
{
    return m_focusedBack;
}

void Button::setSoundEngine(SoundEngine &soundEngine)
{
    m_soundEngine = &soundEngine;
}

const SoundEngine &Button::getSoundEngine() const
{
    return *m_soundEngine;
}

void Button::setFocusSound(const std::string &focusSound)
{
    m_focusSound = focusSound;
}

const std::string &Button::getFocusSound() const
{
    return m_focusSound;
}

void Button::setClickSound(const std::string &clickSound)
{
    m_clickSound = clickSound;
}

const std::string &Button::getClickSound() const
{
    return m_clickSound;
}

float Button::getWidth() const
{
    if (m_focused)
        return m_focusedBack.getWidth() * m_focusedBack.getXScale();
    
    return m_inactiveBack.getWidth() * m_inactiveBack.getXScale();
}

float Button::getHeight() const
{
    if (m_focused)
        return m_focusedBack.getHeight() * m_focusedBack.getYScale();
    
    return m_inactiveBack.getHeight() * m_inactiveBack.getYScale();
}
