/*Copyright (C) 2010-2015 Duncan Deveaux

This file is part of Hikou no mizu.

Hikou no mizu is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Hikou no mizu is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Hikou no mizu.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef DEF_BUTTON
#define DEF_BUTTON

#define NO_SOUND ""

#include "Widget.hpp"
#include "Graphics/Drawable/Sprite.hpp"

#include "Engines/Sound/SoundEngine.hpp"

class Button : public Widget
{
    public:
        Button();
        Button(const Sprite &inactive, const Sprite &focused);
        void draw();
        
        bool mouseClick(const Vector &mouseCoords);
        bool mouseMove(const Vector &mouseCoords);
        
        void setInactiveBack(const Sprite &inactive);
        const Sprite &getInactiveBack() const;
        
        void setFocusedBack(const Sprite &focused);
        const Sprite &getFocusedBack() const;
        
        void setSoundEngine(SoundEngine &soundEngine);
        const SoundEngine &getSoundEngine() const;
        
        void setFocusSound(const std::string &focusSound);
        const std::string &getFocusSound() const;
        
        void setClickSound(const std::string &clickSound);
        const std::string &getClickSound() const;
        
        float getWidth() const;
        float getHeight() const;
    
    protected:
        Sprite m_inactiveBack, m_focusedBack;
        
        SoundEngine *m_soundEngine; //To handle sounds on events
        std::string m_focusSound, m_clickSound;
};

#endif
