/*Copyright (C) 2010-2015 Duncan Deveaux

This file is part of Hikou no mizu.

Hikou no mizu is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Hikou no mizu is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Hikou no mizu.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "Human.hpp"

Human::Human() : Player()
{
    
}

Human::Human(const std::string &name, float health, float strength, float speed, float jumpEnergy, float walkPeriod) :
Player(name, health, strength, speed, jumpEnergy, walkPeriod)
{
    
}

void Human::act()
{
    //Real time key state
    Uint8 *keystate = SDL_GetKeyState(NULL);
    
    //Update moves
    if (m_moves.getAction() == ACTION_DEFAULT && !isKo() && !isHit())
    {
        if (keystate[ m_keys.getMoveLeftKey() ])
        {
            if (m_moves.getState() != STATE_MOVING)
                m_moves.setState(STATE_MOVING);
            
            m_moves.setXFlip(true);
        }
        else if (keystate[ m_keys.getMoveRightKey() ])
        {
            if (m_moves.getState() != STATE_MOVING)
                m_moves.setState(STATE_MOVING);
            
            m_moves.setXFlip(false);
        }
        else if (keystate[ m_keys.getCrouchKey() ])
        {
            if (m_moves.getState() != STATE_CROUCHED)
                crouch();
        }
        else if (m_moves.getState() == STATE_MOVING || m_moves.getState() == STATE_CROUCHED)
            m_moves.setState(STATE_DEFAULT); //Go back to default state
    }
}

void Human::keyDown(const SDL_KeyboardEvent &keyEvent)
{
    //Key state on event
    if (keyEvent.keysym.sym == m_keys.getPunchKey()) punch(); //Other actions
    else if (keyEvent.keysym.sym == m_keys.getKickKey()) kick();
    else if (keyEvent.keysym.sym == m_keys.getJumpKey()) jump();
}

void Human::setKeys(const PlayerKeys &keys)
{
    m_keys = keys;
}

const PlayerKeys &Human::getKeys() const
{
    return m_keys;
}

//Static methods
Human Human::loadFromXML(const std::string &name, const std::string &xmlPath)
{
    //TinyXML initialization
    TiXmlDocument xmlFile;
    xmlFile.LoadFile(xmlPath.c_str());
    
    TiXmlHandle hdl(&xmlFile);
    TiXmlElement *xmlElement = hdl.FirstChildElement().FirstChildElement().Element();
    
    int i = 0;
    while (xmlElement)
    {
        std::string itemName = xmlElement->Attribute("name");
        if (itemName == name)
        {
            //Get informations
            float strenght = 10, speed = 700, jumpEnergy = 750, walkPeriod = .20;
            
            xmlElement->QueryFloatAttribute("strenght", &strenght);
            xmlElement->QueryFloatAttribute("speed", &speed);
            xmlElement->QueryFloatAttribute("jumpEnergy", &jumpEnergy);
            xmlElement->QueryFloatAttribute("walkPeriod", &walkPeriod);
            
            //Create player
            return Human(name, 100, strenght, speed, jumpEnergy, walkPeriod);
        }
        
        xmlElement = xmlElement->NextSiblingElement();
        i++;
    }
    
    return Human();
}
