/*Copyright (C) 2010-2012 Duncan Deveaux

This file is part of Hikou no mizu.

Hikou no mizu is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Hikou no mizu is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Hikou no mizu.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "Characters.hpp"
#include "Engines/GameEngine.hpp"

Characters::Characters(GameEngine *gameEngine) :
m_gameEngine(gameEngine), m_window(gameEngine->getWindow()), m_playersNo(2)
{
    
}

void Characters::setPlayersNo(int playersNo)
{
    m_playersNo = playersNo;
}

//Screen virtual method
void Characters::run()
{
    //Init variables
        //Get view size
    Vector viewSize = Vector(m_window->getViewWidth(), m_window->getViewHeight());
    float viewScale = viewSize.y / 1440;
    
    //Interface
    TextureManager textureManager;
    FontManager fontManager;
    
    Texture *uiTexture = textureManager.getTexture("data/gfx/ui/chooser_ui.png");
    
    //Blue & red sections
    Sprite blueSection((*uiTexture), Box(20.f / 4096, 20.f / 2048, 890.f / 4096, 1440.f / 2048));
    blueSection.setScale(viewScale);
    blueSection.setXOrigin(blueSection.getWidth());
    
    Sprite redSection((*uiTexture), Box(922.f / 4096, 20.f / 2048, 890.f / 4096, 1440.f / 2048));
    redSection.setScale(viewScale);
    redSection.setXPosition(viewSize.x);
    
    //Background & Title
    Polygon background = Polygon::rectangle(-4, -4, viewSize.x + 8, viewSize.y + 8, Color(56, 56, 56));
    background.setBorderSize(0);
    
    Sprite title((*uiTexture), Box(1832.f / 4096, 10.f / 2048, 970.f / 4096, 300.f / 2048));
    title.setScale(viewScale);
    
    title.setXPosition(blueSection.getWidth() * blueSection.getXScale() + 2);
    title.setYPosition(1);
    
    //Item test
    //Load animations & aliases
    std::map<unsigned int, Animation> anims;
    anims[NO_SELECT].setTexture((*uiTexture));
    anims[NO_SELECT].addFrame( Frame( Box( 2320, 320, 271, 521 ), Box(0, 0, 271, 521) ) );
    Characters::load(anims, textureManager);
    
    std::map<unsigned int, std::string> aliases;
    aliases[NO_SELECT] = "";
    aliases[SELECT_HIKOU] = "Hikou";
    aliases[SELECT_TAKINO] = "Takino";
    aliases[SELECT_RANDOM] = "Random";
    
    //Items
    Sprite back_item((*uiTexture), Box(1832.f / 4096, 320.f / 2048, 478.f / 4096, 562.f / 2048));
    Sprite backbutton_item((*uiTexture), Box(2601.f / 4096, 533.f / 2048, 147.f / 4096, 147.f / 2048));
    
    TextButton button_item(backbutton_item, backbutton_item);
    button_item.setTextFont( *fontManager.getFont("data/gfx/fonts/default.ttf", 40) );
    button_item.setTextColor( Color(255, 255, 255) );
    
    std::vector< std::pair<PlayerItem, unsigned int> > items;
    
    bool leftSide = true;
    int v_factor = 0;
    for (int i = 0; i < m_playersNo; i++)
    {
        PlayerItem item(478, 562, back_item, i + 1);
        item.changePlayer(anims[NO_SELECT], aliases[NO_SELECT]);
        
        item.setTextFont( *fontManager.getFont("data/gfx/fonts/default.ttf", 50) );
        item.setModeButton(button_item);
    
        item.setScale(viewScale);
        
        //Equation for left-side items' position (see .xcf data file) : y = ax + b
        //With A(157,65) and B(87,738) two points from the line :
        //| 65 = 157a + b   | a = -9.61
        //| 738 = 87a + b   | b = 1574
        //Then : y = -9.61x + 1574 <=> x = -.104(y - 1574)
        
        //In the same way, the equations for right-side items are :
        // y = -9.61x + 19075 <=> x = -.104(y - 19075)
        
        //We scale & use the equations we simplified to set the item's position
        int h_size = static_cast<int>((m_playersNo + 1) / 2);
        float space = (viewSize.y - (item.getHeight() * viewScale * (1.2 * h_size - .2)) ) / 2;
        
        if (leftSide) //Left side equation
        {
            item.setYPosition(space + (-39 + 1.2 * item.getHeight() * v_factor) * viewScale);
            item.setXPosition(-.104 * ((65 + 1.2 * item.getHeight() * v_factor) - 1574) * viewScale);
        }
        else //Right side equation
        {
            item.setYPosition(space + (39 + 1.2 * item.getHeight() * v_factor) * viewScale);
            item.setXPosition(-.104 * ((143 + 1.2 * item.getHeight() * v_factor) - 19075) * viewScale);
        }
        
        //Add the item to items
        items.push_back( std::make_pair(item, NO_SELECT) );
        
        //Switch side
        leftSide = !leftSide;
        
        if (leftSide)
            v_factor++;
    }
    
    //Chooser
    Sprite inactive_chooser((*uiTexture), Box(1832.f / 4096, 892.f / 2048, 233.f / 4096, 233.f / 2048));
    Sprite focused_chooser((*uiTexture), Box(2075.f / 4096, 892.f / 2048, 233.f / 4096, 233.f / 2048));
    
    PlayerChooser chooser(fontManager, inactive_chooser, focused_chooser);
    chooser.setScale(viewScale);
    chooser.setPosition(title.getXPosition() - 2, 0.3 * viewSize.y); //Center of the view (y)
    
    //Ok button
    Sprite back_ok((*uiTexture), Box(1832.f / 4096, 1135.f / 2048, 500.f / 4096, 172.f / 2048));
    
    TextButton validate("OK", back_ok, back_ok);
    validate.setTextFont( *fontManager.getFont("data/gfx/fonts/default.ttf", 107) );
    validate.setTextColor( Color(255, 255, 255) );
    
    validate.setScale(viewScale);
    
    validate.setXPosition(chooser.getXPosition() + (chooser.getWidth() * chooser.getXScale() - validate.getWidth() * validate.getXScale()) / 2);
    validate.setYPosition(chooser.getYPosition() + chooser.getHeight() * chooser.getYScale() + .08 * viewSize.y);
    
    //Start screen loop
    Vector mouse;
    
    Timer time;
    float frameTime = 0;
    bool introEnded = false;
    int focused = 0;
    
    SDL_Event event;
    while (1)
    {
        //Manage time
        frameTime = time.getTime();
        time.reset();
        
        //Manage events
        while (m_window->pollEvent(event))
        {
            switch (event.type)
            {
                case SDL_QUIT: //Application closed
                    m_gameEngine->exit();
                    return;
                break;
                
                case SDL_VIDEORESIZE: //Window resized
                    m_window->resize(event.resize.w, event.resize.h, false);
                break;
                
                case SDL_KEYDOWN: //Key pressed
                    if (event.key.keysym.sym == SDLK_ESCAPE)
                    {
                        m_gameEngine->exit();
                        return;
                    }
                break;
                
                case SDL_MOUSEBUTTONUP: //Mouse clicked
                    mouse.x = event.button.x;
                    mouse.y = event.button.y;
                    
                    if (introEnded)
                    {
                        //Update Human/AI buttons
                        std::vector< std::pair<PlayerItem, unsigned int> >::iterator it;
                        for (it = items.begin(); it != items.end(); it++)
                            it->first.mouseClick(m_window->viewCoords(mouse));
                        
                        //Get selection & update focused item
                        if (focused != -1)
                        {
                            unsigned int selection = NO_SELECT;
                            chooser.mouseClick(m_window->viewCoords(mouse), selection);
                            
                            if (selection != NO_SELECT)
                            {
                                if (items[focused].first.getAlias() != aliases[selection])
                                    items[focused].first.changePlayer(anims[selection], aliases[selection]);
                                items[focused].second = selection;
                                
                                if (focused < m_playersNo - 1)
                                    focused++;
                                else
                                    focused = -1;
                                
                                //std::cout << focused << std::endl;
                            }
                        }
                    }
                break;
                
                case SDL_MOUSEMOTION: //Mouse moved
                    mouse.x = event.motion.x;
                    mouse.y = event.motion.y;
                    
                    if (introEnded)
                    {
                        //Get selection & update focused item
                        if (focused != -1)
                        {
                            unsigned int selection = NO_SELECT;
                            if (!chooser.mouseMove(m_window->viewCoords(mouse), selection)) //Mouse out of the box
                            {
                                Animation &updated_anim = anims[ items[focused].second ];
                                std::string &updated_alias = aliases[ items[focused].second ];
                                
                                if (items[focused].first.getAlias() != updated_alias)
                                    items[focused].first.changePlayer(updated_anim, updated_alias);
                            }
                            else if (selection != NO_SELECT && focused != -1)
                            {
                                Animation &updated_anim = anims[selection];
                                std::string &updated_alias = aliases[selection];
                                
                                if (items[focused].first.getAlias() != updated_alias)
                                    items[focused].first.changePlayer(updated_anim, updated_alias);
                            }
                        }
                    }
                break;
            }
        }
        
        //Intro
        if (!introEnded)
        {
            //The intro is 0.4 second long.
            float elapsedTime = (frameTime / 1000);
            float deltaMove = 2.5 * blueSection.getWidth() * elapsedTime;
            
            if (blueSection.getXOrigin() - deltaMove > 0)
            {
                blueSection.setXOrigin(blueSection.getXOrigin() - deltaMove);
                redSection.setXOrigin(redSection.getXOrigin() + deltaMove);
            }
            else
            {
                blueSection.setXOrigin(0);
                redSection.setXOrigin( redSection.getWidth() );
                
                introEnded = true;
            }
        }
        
        if (introEnded)
        {
            //Update items (animations)
            std::vector< std::pair<PlayerItem, unsigned int> >::iterator it;
            for (it = items.begin(); it != items.end(); it++)
                it->first.update(frameTime);
        }
        
        //Display
            //Clear
        m_window->clear();
        
            //Draw
        background.draw();
        
        blueSection.draw();
        redSection.draw();
        
        title.draw();
        if (introEnded)
        {
            //Draw items
            std::vector< std::pair<PlayerItem, unsigned int> >::iterator it;
            for (it = items.begin(); it != items.end(); it++)
                it->first.draw();
            
            chooser.draw();
        }
        
        validate.draw();
        
            //Render
        m_window->flush();
    }
}

void Characters::load(std::map<unsigned int, Animation> &target, TextureManager &manager) //Load animations
{
    //Load characters' animations
    loadAnimation("Hikou", target[SELECT_HIKOU], manager);
    loadAnimation("Takino", target[SELECT_TAKINO], manager);
}

void Characters::loadAnimation(const std::string &player, Animation &dst, TextureManager &manager) const
{
    std::string path = "data/gfx/characters/" + player + "/data.xml";
    
    //TinyXML initialization
    TiXmlDocument xmlFile;
    xmlFile.LoadFile(path.c_str());
    
    TiXmlHandle hdl(&xmlFile);
    hdl = hdl.FirstChildElement().FirstChildElement();
    
    //Default state -> default animation
    TiXmlElement *xmlAnimation = hdl.FirstChildElement().FirstChildElement("animation").ToElement();
    if (xmlAnimation != NULL)
    {
        //Load texture
        std::string animationPath = "data/gfx/characters/" + player + "/" + std::string(xmlAnimation->Attribute("path")) + ".png";
        Texture *texture = manager.getTexture(animationPath);
        
        //Main params
        float duration = 0;
        xmlAnimation->QueryFloatAttribute("duration", &duration);
        
        dst.setTexture((*texture));
        dst.setDuration(duration);
        
        //Frames
        if (!xmlAnimation->NoChildren())
        {
            const TiXmlElement *xmlFrames = xmlAnimation->FirstChildElement();
            
            while (xmlFrames)
            {
                int srcX = 0, srcY = 0, srcWidth = 0, srcHeight = 0, time = 1;
                float bodyX = 0, bodyY = 0, bodyWidth = 0, bodyHeight = 0;
                
                xmlFrames->QueryIntAttribute("x", &srcX);
                xmlFrames->QueryIntAttribute("y", &srcY);
                
                xmlFrames->QueryIntAttribute("width", &srcWidth);
                xmlFrames->QueryIntAttribute("height", &srcHeight);
                
                xmlFrames->QueryFloatAttribute("bodyX", &bodyX);
                xmlFrames->QueryFloatAttribute("bodyY", &bodyY);
                
                xmlFrames->QueryFloatAttribute("bodyWidth", &bodyWidth);
                xmlFrames->QueryFloatAttribute("bodyHeight", &bodyHeight);
                
                xmlFrames->QueryIntAttribute("time", &time);
                
                Frame frame( Box(srcX, srcY, srcWidth, srcHeight), Box(bodyX, bodyY, bodyWidth, bodyHeight) );
                
                for (int i = 0; i < time; i++)
                    dst.addFrame(frame);
                
                xmlFrames = xmlFrames->NextSiblingElement();
            }
        }
    }
}
