/*Copyright (C) 2010-2012 Duncan Deveaux

This file is part of Hikou no mizu.

Hikou no mizu is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Hikou no mizu is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Hikou no mizu.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "PlayerChooser.hpp"

PlayerChooser::PlayerChooser(FontManager &fontManager, const Sprite &inactive, const Sprite &focused) :
m_inactive(inactive), m_focusedBack(focused)
{
    //Background
    m_background = Polygon::rectangle(0, 0, 775, 600, Color(0, 0, 0, 54));
    m_background.setBorderSize(0);
    
    //Buttons
    m_hikou = TextButton("Hi", m_inactive, m_focusedBack);
    m_takino = TextButton("Ta", m_inactive, m_focusedBack);
    m_random = TextButton("Ra", m_inactive, m_focusedBack);
    
    m_hikou.setTextFont( *fontManager.getFont("data/gfx/fonts/default.ttf", 100) );
    m_takino.setTextFont( *fontManager.getFont("data/gfx/fonts/default.ttf", 100) );
    m_random.setTextFont( *fontManager.getFont("data/gfx/fonts/default.ttf", 100) );
    
    m_hikou.setTextColor( Color(255, 255, 255) );
    m_takino.setTextColor( Color(255, 255, 255) );
    m_random.setTextColor( Color(255, 255, 255) );
    
    //Position
    float v_space = 74, h_space = 41; //Spacing between the buttons
    
    //x = totalwidth - x - buttonwidth - v_space - buttonwidth
    // <=> 2x = totalWidth - 2buttonwidth - v_space
    // <=> x = 1/2 (totalWidth - v_space) - buttonwidth
    
    //y = totalheight - y - buttonheight - h_space - buttonheight
    // <=> ... (see above)
    
    m_hikou.setXPosition((getWidth() - v_space) / 2 - m_hikou.getWidth());
    m_hikou.setYPosition((getHeight() - h_space) / 2 - m_hikou.getHeight());
    
    m_takino.setXPosition(m_hikou.getXPosition() + m_hikou.getWidth() + v_space);
    m_takino.setYPosition(m_hikou.getYPosition());
    
    m_random.setXPosition(m_hikou.getXPosition() + (m_hikou.getWidth() + v_space) / 2);
    m_random.setYPosition(m_hikou.getYPosition() + m_hikou.getHeight() + h_space);
}

void PlayerChooser::draw()
{
    //Update matrix
    Drawable::pushMatrix();
    Drawable::updateMatrix();
    
    //Draw
    m_background.draw();
    
    m_hikou.draw();
    m_takino.draw();
    m_random.draw();
    
    Drawable::popMatrix();
}

//Widget methods
bool PlayerChooser::mouseClick(const Vector &mouseCoords)
{
    //Calls the other mouseClick() ;)
    unsigned int selection = 0;
    return mouseClick(mouseCoords, selection);
}

bool PlayerChooser::mouseMove(const Vector &mouseCoords)
{
    //Calls the other mouseMove() ;)
    unsigned int selection = 0;
    return mouseMove(mouseCoords, selection);
}

//Widget methods : Returns the updated player
bool PlayerChooser::mouseClick(const Vector &mouseCoords, unsigned int &target)
{
    Vector localMouse = Widget::localCoords(mouseCoords);
    
    if (m_hikou.mouseClick(localMouse))
        target = SELECT_HIKOU;
    else if (m_takino.mouseClick(localMouse))
        target = SELECT_TAKINO;
    else if (m_random.mouseClick(localMouse))
        target = SELECT_RANDOM;
    else
        target = NO_SELECT;
    
    return Widget::contains(localMouse);
}

bool PlayerChooser::mouseMove(const Vector &mouseCoords, unsigned int &target)
{
    Vector localMouse = Widget::localCoords(mouseCoords);
    
    if (m_hikou.mouseMove(localMouse))
        target = SELECT_HIKOU;
    else if (m_takino.mouseMove(localMouse))
        target = SELECT_TAKINO;
    else if (m_random.mouseMove(localMouse))
        target = SELECT_RANDOM;
    else
        target = NO_SELECT;
    
    return Widget::contains(localMouse);
}

float PlayerChooser::getWidth() const
{
    return 775;
}

float PlayerChooser::getHeight() const
{
    return 600;
}
