/*Copyright (C) 2010-2012 Duncan Deveaux

This file is part of Hikou no mizu.

Hikou no mizu is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Hikou no mizu is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Hikou no mizu.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef DEF_PLAYER_ITEM
#define DEF_PLAYER_ITEM

#define MODE_HUMAN 0
#define MODE_AI 1

#include "Animation/Animation.hpp"
#include "Graphics/Drawable/Sprite.hpp"
#include "Graphics/Drawable/Text.hpp"
#include "GUI/Widget/TextButton.hpp"

#include <sstream>
#include <string>

class PlayerItem : public Widget
{
    public:
        PlayerItem();
        PlayerItem(float width, float height, const Sprite &background, unsigned int chooserId);
        void draw();
        
        //Widget methods
        bool mouseClick(const Vector &mouseCoords);
        bool mouseMove(const Vector &mouseCoords);
        
        void update(float frameTime);
        void changePlayer(const Animation &animation, const std::string &alias);
        
        void setChooserId(unsigned int chooserId);
        unsigned int getChooserId() const;
        
        void setWidth(float width);
        float getWidth() const;
        
        void setHeight(float height);
        float getHeight() const;
        
        void setBackground(const Sprite &background);
        const Sprite &getBackground() const;
        
        void setTextFont(Font &textFont);
        const Font &getTextFont() const;
        
        void setModeButton(const TextButton &modeButton);
        const TextButton &getModeButton() const;
        
        const std::string &getAlias() const;
        unsigned int getMode() const;
    
    private:
        std::string m_alias;
        unsigned int m_mode;
        
        Sprite m_background;
        float m_width, m_height;
        
        Animation m_animation;
        Text m_aliasText, m_idText;
        TextButton m_modeButton;
        
        unsigned int m_chooserId;
};

#endif
