/*Copyright (C) 2010-2015 Duncan Deveaux

This file is part of Hikou no mizu.

Hikou no mizu is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Hikou no mizu is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Hikou no mizu.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "MainMenu.hpp"
#include "Engines/GameEngine.hpp"

MainMenu::MainMenu(GameEngine *gameEngine) :
m_gameEngine(gameEngine), m_window(gameEngine->getWindow()),
m_configuration(gameEngine->getConfiguration())
{
    
}

void MainMenu::run()
{
    //Init variables
        //Get view size
    Vector viewSize = Vector(m_window->getViewWidth(), m_window->getViewHeight());
    
    TextureManager textureManager;
    FontManager fontManager;
    SoundBufferManager soundBufferManager;
    
        //Background ( + Title)
    Sprite background( (*textureManager.getTexture("data/gfx/ui/title.png")), Box(0, 0, 1280.f / 2048, 720.f / 1024) );
    background.setScale(viewSize.x / background.getWidth(), viewSize.y / background.getHeight());
    
    Sprite title( (*textureManager.getTexture("data/gfx/ui/banner.png")), Box(20.f / 2048, 20.f / 1024, 1789.f / 2048, 305.f / 1024) );
    title.setScale(.4);
    title.setPosition((viewSize.x - title.getWidth() * title.getXScale()) / 2, 30);
    
        //Menu
    SoundEngine soundEngine;
    soundEngine.setManager(soundBufferManager);
    soundEngine.setVolume(m_configuration->getMasterVolume() * m_configuration->getSoundVolume() / 100.f);
    
    Sprite menu_default( (*textureManager.getTexture("data/gfx/ui/ui.png")), Box(0, 0, 240.f / 512, 60.f / 512) ),
    menu_hover( (*textureManager.getTexture("data/gfx/ui/ui.png")), Box(0, 64.f / 512, 240.f / 512, 60.f / 512) );
    
    TextButton menuTheme(menu_default, menu_hover);
    menuTheme.setSoundEngine(soundEngine);
    menuTheme.setFocusSound("data/sound/ui/menuFocus.ogg");
    
    menuTheme.setTextFont(*fontManager.getFont("data/gfx/fonts/default.ttf", 30));
    menuTheme.setTextColor(Color(255, 255, 255));
    
    TextMenu menu(menuTheme, 20, 0, 275, 350, Color(66, 112, 174));
    menu.loadData("data/cfg/menus/mainMenu.xml");
    menu.launch();
    
    menu.setXPosition((viewSize.x - menu.getWidth()) / 10);
    menu.setYPosition(title.getYPosition() + title.getHeight() * title.getYScale() + 30);
    
    //Start the playlist
    MusicPlayer::stopMusic("data/sound/music/S31-Unexpected Trouble.ogg");
    
    Playlist playlist;
    playlist.setVolume( m_configuration->getMasterVolume() * m_configuration->getMusicVolume() / 100.f );
    playlist.addSong( SongData("A Cog Life", "epopsan", "data/sound/music/menu/Epopsan_-_______a_cog_life_.ogg") );
    playlist.addSong( SongData("Blue Sky Guitar", "epopsan", "data/sound/music/menu/Epopsan_-_________blue_sky_guitar_.ogg") );
    playlist.addSong( SongData("Restart", "epopsan", "data/sound/music/menu/Epopsan_-_______restart_.ogg") );
    playlist.addSong( SongData("Signal", "epopsan", "data/sound/music/menu/Epopsan_-______signal_.ogg") );
    playlist.addSong( SongData("A Glimmer Of Light (instr.)", "epopsan", "data/sound/music/menu/epopsan2/Epopsan_-_a_glimmer_of_light__instrumental_.ogg") );
    playlist.addSong( SongData("People And Particles", "epopsan", "data/sound/music/menu/epopsan2/Epopsan_-_People_and_particles__________.ogg") );
    playlist.addSong( SongData("Plate", "epopsan", "data/sound/music/menu/epopsan2/Epopsan_-_Plate______.ogg") );
    playlist.addSong( SongData("Reminiscences Listener", "epopsan", "data/sound/music/menu/epopsan2/Epopsan_-_Reminiscences_listener_________.ogg") );
    playlist.addSong( SongData("The Song For Human Beings", "epopsan", "data/sound/music/menu/epopsan2/Epopsan_-_The_song_for_human_beings______.ogg") );
    playlist.addSong( SongData("Those Who Walk (instr.)", "epopsan", "data/sound/music/menu/epopsan2/Epopsan_-_Those_who_walk_instrumental_.ogg") );
    playlist.addSong( SongData("Universal Orbit", "epopsan", "data/sound/music/menu/epopsan2/Epopsan_-_Universal_orbit______.ogg") );
    playlist.addSong( SongData("Voice", "epopsan", "data/sound/music/menu/epopsan2/Epopsan_-_Voice_______.ogg") );
    
    
    pthread_mutex_t notifierMutex = PTHREAD_MUTEX_INITIALIZER;
    
    PlaylistNotifier notifier;
    notifier.setNotifyTime(2000.f);
    notifier.setFonts(*fontManager.getFont("data/gfx/fonts/default.ttf", 20), *fontManager.getFont("data/gfx/fonts/default.ttf", 12));
    
    notifier.setPosition( viewSize.x - notifier.getWidth() - 20, viewSize.y - notifier.getHeight() - 20 );
    
    
    if (!PlaylistPlayer::startPlaylist(playlist, PLAYLIST_MENU, &notifier, &notifierMutex))
        PlaylistPlayer::playlistSetNotifier(PLAYLIST_MENU, &notifier, &notifierMutex);
    
    //Start screen loop
    Vector mouse;
    SDL_Event event;
    while (1)
    {
        //Manage events
        while (m_window->pollEvent(event))
        {
            switch (event.type)
            {
                case SDL_QUIT: //Application closed
                    m_gameEngine->exit();
                    
                    PlaylistPlayer::playlistSetNotifier(PLAYLIST_MENU, NULL, NULL);
                    pthread_mutex_destroy(&notifierMutex);
                    return;
                break;
                
                case SDL_VIDEORESIZE: //Window resized
                    m_window->resize(event.resize.w, event.resize.h, false);
                break;
                
                case SDL_MOUSEBUTTONUP: //Mouse clicked
                    mouse.x = event.button.x;
                    mouse.y = event.button.y;
                    
                    menu.mouseClick(m_window->viewCoords(mouse));
                break;
                
                case SDL_MOUSEMOTION: //Mouse moved
                    mouse.x = event.motion.x;
                    mouse.y = event.motion.y;
                    
                    menu.mouseMove(m_window->viewCoords(mouse));
                break;
            }
        }
            
        if (menu.browsed()) //Browsing ended
        {
            std::string result = menu.result().back();
            
            if (result == "legacy") //Want to play the game
            {
                m_gameEngine->confCharactersMenu_playersNo(2);
                m_gameEngine->initCharactersMenu();
                break;
            }
            else if (result == "chooser")
            {
                m_gameEngine->confCharacters_playersNo(2);
                m_gameEngine->initCharacters();
                break;
            }
            else if (result == "options")
            {
                m_gameEngine->initOptions();
                break;
            }
            else if (result == "exit") //Want to exit
            {
                m_gameEngine->exit();
                break;
            }
            
            //Reload menu
            menu.launch();
        }
        
        //Display
            //Clear
        m_window->clear();
        
            //Draw
        background.draw();
        title.draw();
        menu.draw();
        
            //Notifier
        pthread_mutex_lock(&notifierMutex);
        
        notifier.update();
        notifier.draw();
        
        pthread_mutex_unlock(&notifierMutex);
        
            //Render
        m_window->flush();
    }
    
    PlaylistPlayer::playlistSetNotifier(PLAYLIST_MENU, NULL, NULL);
    pthread_mutex_destroy(&notifierMutex);
}
