/*Copyright (C) 2010-2016 Duncan Deveaux

This file is part of Hikou no mizu.

Hikou no mizu is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Hikou no mizu is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Hikou no mizu.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "KeyBindTab.hpp"

KeyBindTab::KeyBindTab() : Tab(), m_configuration(NULL), m_playerIx(0)
{
    
}

KeyBindTab::KeyBindTab(Configuration &configuration, unsigned int playerIx, TextureManager &manager, Font &titleFont, Font &labelFont, SoundEngine &soundEngine) :
Tab(375, 250, Color(50, 93, 151)), m_configuration(&configuration), m_playerIx(playerIx)
{
    Sprite chooser_default( (*manager.getTexture("data/gfx/ui/ui.png")), Box(0, 0, 240.f / 512, 60.f / 512) ),
    chooser_hover( (*manager.getTexture("data/gfx/ui/ui.png")), Box(0, 64.f / 512, 240.f / 512, 60.f / 512) );
    chooser_default.setScale(.3, .5);
    chooser_hover.setScale(.3, .5);
    
    //Title text & apply Button
    std::ostringstream titleStr;
    titleStr << "Player #" << m_playerIx << " key binder";
    
    m_titleText.setText(titleStr.str());
    m_titleText.setFont(titleFont);
    m_titleText.setPosition( (getWidth() - m_titleText.getWidth()) / 2, 10);
    
    m_applyButton = TextButton("Apply !", chooser_default, chooser_hover);
    m_applyButton.setTextFont(labelFont);
    m_applyButton.setSoundEngine(soundEngine);
    m_applyButton.setFocusSound("data/sound/ui/menuFocus.ogg");
    m_applyButton.setClickSound("data/sound/ui/menuClick.ogg");
    
    m_applyButton.setPosition( getWidth() - m_applyButton.getWidth() - 10, getHeight() - m_applyButton.getHeight() - 10);
    
    //Key binders
    float yBase = m_titleText.getYPosition() + m_titleText.getHeight() + 20;
    chooser_default.setScale(.3, .4);
    chooser_hover.setScale(.3, .4);
    
    PlayerKeys initialKeys; //Keys to initialize the Key Binders
    if (m_configuration->getPlayerKeys(m_playerIx, initialKeys))
    {
        m_punch = KeyLabel( "Punch :", labelFont, KeyBinder( initialKeys.getPunchKey() ), 10, 100, yBase, chooser_default, chooser_hover, soundEngine, "data/sound/ui/menuFocus.ogg", "data/sound/ui/menuClick.ogg" );
        m_kick = KeyLabel( "Kick :", labelFont, KeyBinder( initialKeys.getKickKey() ), 10, 100, yBase + 30, chooser_default, chooser_hover, soundEngine, "data/sound/ui/menuFocus.ogg", "data/sound/ui/menuClick.ogg" );
        m_jump = KeyLabel( "Jump :", labelFont, KeyBinder( initialKeys.getJumpKey() ), 10, 100, yBase + 60, chooser_default, chooser_hover, soundEngine, "data/sound/ui/menuFocus.ogg", "data/sound/ui/menuClick.ogg" );
        m_moveLeft = KeyLabel( "Move left :", labelFont, KeyBinder( initialKeys.getMoveLeftKey() ), 10, 100, yBase + 90, chooser_default, chooser_hover, soundEngine, "data/sound/ui/menuFocus.ogg", "data/sound/ui/menuClick.ogg" );
        m_moveRight = KeyLabel( "Move right :", labelFont, KeyBinder( initialKeys.getMoveRightKey() ), 10, 100, yBase + 120, chooser_default, chooser_hover, soundEngine, "data/sound/ui/menuFocus.ogg", "data/sound/ui/menuClick.ogg" );
        m_crouch = KeyLabel( "Crouch :", labelFont, KeyBinder( initialKeys.getCrouchKey() ), 10, 100, yBase + 150, chooser_default, chooser_hover, soundEngine, "data/sound/ui/menuFocus.ogg", "data/sound/ui/menuClick.ogg" );
    }
}

void KeyBindTab::draw()
{
    //Update matrix
    Drawable::pushMatrix();
    Drawable::updateMatrix();
    
    m_background.draw();
    
    //Draw title text
    m_titleText.draw();
    
    //Draw binders
    m_punch.draw();
    m_kick.draw();
    m_jump.draw();
    m_moveLeft.draw();
    m_moveRight.draw();
    m_crouch.draw();
    
    //Draw "apply"
    m_applyButton.draw();
    
    Drawable::popMatrix();
}

bool KeyBindTab::mouseClick(const Vector &mouseCoords)
{
    Vector localMouse = Widget::localCoords(mouseCoords);
    
    m_punch.binder.mouseClick(localMouse);
    m_kick.binder.mouseClick(localMouse);
    m_jump.binder.mouseClick(localMouse);
    m_moveLeft.binder.mouseClick(localMouse);
    m_moveRight.binder.mouseClick(localMouse);
    m_crouch.binder.mouseClick(localMouse);
    
    if (m_applyButton.mouseClick(localMouse))
        apply();
    
    return Widget::contains(localMouse);
}

bool KeyBindTab::mouseMove(const Vector &mouseCoords)
{
    Vector localMouse = Widget::localCoords(mouseCoords);
    m_applyButton.mouseMove(localMouse);
    
    return Widget::contains(localMouse);
}

void KeyBindTab::keyDown(const SDLKey &key)
{
    m_punch.binder.keyDown(key);
    m_kick.binder.keyDown(key);
    m_jump.binder.keyDown(key);
    m_moveLeft.binder.keyDown(key);
    m_moveRight.binder.keyDown(key);
    m_crouch.binder.keyDown(key);
}

void KeyBindTab::apply()
{
    PlayerKeys newKeys(m_punch.binder.getKey(), m_kick.binder.getKey(), m_jump.binder.getKey(),
    m_moveLeft.binder.getKey(), m_moveRight.binder.getKey(), m_crouch.binder.getKey());
    
    if (m_configuration != NULL)
        m_configuration->setPlayerKeys(m_playerIx, newKeys);
}
