/*Copyright (C) 2010-2016 Duncan Deveaux

This file is part of Hikou no mizu.

Hikou no mizu is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Hikou no mizu is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Hikou no mizu.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef DEF_KEY_BIND_TAB
#define DEF_KEY_BIND_TAB

#include "Configuration/Configuration.hpp"
#include "Engines/Resources/TextureManager.hpp"
#include "Engines/Sound/SoundEngine.hpp"

#include "GUI/Widget/Tab.hpp"
#include "GUI/Widget/KeyBinder.hpp"
#include "GUI/Widget/TextButton.hpp"

#include <sstream>

///A key binding label : Text label & the associated key binder
struct KeyLabel
{
    KeyLabel()
    {
        
    }
    
    KeyLabel(const std::string &labelText, Font &labelFont, const KeyBinder &keyBinder,
    float xBase, float xGap, float yBase, const Sprite &inactiveBack, const Sprite &focusedBack,
    SoundEngine &soundEngine, const std::string &focusSound, const std::string &keySetSound) :
    binder(keyBinder)
    {
        binder.setPosition(xBase + xGap, yBase);
        
        binder.setTextFont(labelFont);
        binder.setInactiveBack(inactiveBack);
        binder.setFocusedBack(focusedBack);
        
        binder.setSoundEngine(soundEngine);
        binder.setFocusSound(focusSound);
        binder.setKeySetSound(keySetSound);
        
        label.setText(labelText);
        label.setFont(labelFont);
        
        label.setPosition(xBase, yBase + (binder.getHeight() - label.getHeight()) / 2);
    }
    
    void draw()
    {
        label.draw();
        binder.draw();
    }
    
    Text label; //A key label
    KeyBinder binder; //Its associated key binder
};

///KeyBindTab : tab interface to set the key bindings of a player
///(of a given index), includes the 'apply' button which applies the new
///bindings to a Configuration set.
class KeyBindTab : public Tab
{
    public:
        KeyBindTab();
        KeyBindTab(Configuration &configuration, unsigned int playerIx, TextureManager &manager, Font &titleFont, Font &labelFont, SoundEngine &soundEngine);
        void draw();
        
        bool mouseClick(const Vector &mouseCoords);
        bool mouseMove(const Vector &mouseCoords);
        void keyDown(const SDLKey &key);
        
    private:
        void apply();
        
        //Title message
        Text m_titleText;
        
        //Key binders
        KeyLabel m_punch, m_kick;
        KeyLabel m_jump;
        KeyLabel m_moveLeft, m_moveRight;
        KeyLabel m_crouch;
        
        //Apply button
        TextButton m_applyButton;
        
        Configuration *m_configuration;
        unsigned int m_playerIx;
};

#endif
